:- module client.

:- interface.

:- import_module io.

:- pred main(io, io).
:- mode main(di, uo) is det.

%--------------------------------------------------------------------%

:- implementation.

:- import_module list, require, int, string.
:- import_module world, stream.

:- import_module posix.
:- import_module posix__hostent.
:- import_module posix__socket.
:- import_module posix__stream.

main -->
    command_line_arguments(Args),
    ( if { Args = [Server,Port0], to_int(Port0, Port) } then
	main2(Server, Port)
    else
	write_string("Usage: client SERVER PORT\n")
    ).

:- pred main2(string, int, io, io).
:- mode main2(in, in, di, uo) is det.

main2(Server, Port) -->
    gethostbyname(Server, Res0),
    ( if { Res0 = ok(Hostent) } then
	socket(inet, stream, protocol(0), Res1),
	(
	    { Res1 = ok(Fd) },
	    { if Hostent ^ h_addr_list = [Addr0|_] then
		Addr = Addr0
	    else
		error("No address?")
	    },
	    connect(Fd, inet(port(Port), Addr), Res2),
	    (
		{ Res2 = ok },
		main(Fd)
	    ;
		{ Res2 = error(_Error2), error("could not connect") }
	    )
	;
	    { Res1 = error(_Error1), error("could not create socket") }
	)
    else
	{ error("could not resolve host: "++Server) }
    ).
    
:- pred main(fd, io, io).
:- mode main(in, di, uo) is det.

main(Fd) -->
    create_input(Fd, In0),
    putback_stream(In0, In),
    create_output(Fd, Out),
    stream__write_string(Out, "Player\n"),
    read_world(In, World),
    go(In, Out, World, _).

