:- module posix__buffer.		% Copyright (C) 2002 YesLogic Pty. Ltd.

:- interface.

:- import_module char.

:- type buffer
    --->    buffer(size :: int, buffer :: pointer).

:- type pointer.
:- pragma foreign_type(c, pointer, "void *").

% create(Size, Buffer)
:- pred create(int, buffer, io, io).
:- mode create(in, out, di, uo) is det.

% strcpy(DestBuffer, SrcString)
:- pred strcpy(buffer, string, io, io).
:- mode strcpy(in, in, di, uo) is det.

% alloc(Size, Buffer)
:- pred alloc(int, pointer, io, io).
:- mode alloc(in, out, di, uo) is det.

% index(Index, Buffer, Char)
:- pred index(int, pointer, char, io, io).
:- mode index(in, in, out, di, uo) is det.

% set(Index, Buffer, Char)
:- pred set(int, pointer, char, io, io).
:- mode set(in, in, in, di, uo) is det.

% offset(Index, Buffer0, Buffer)
:- pred offset(int, pointer, pointer, io, io).
:- mode offset(in, in, out, di, uo) is det.

%--------------------------------------------------------------------%

:- implementation.

create(Size, buffer(Size, Buffer)) -->
    alloc(Size, Buffer).

:- pragma foreign_proc(c, alloc(Size::in, Buffer::out, IO0::di, IO::uo),
	[will_not_call_mercury, thread_safe, promise_pure], "
    MR_incr_hp_atomic(Buffer, (Size + sizeof(MR_Word) - 1) / sizeof(MR_Word));
    IO = IO0;
").

%--------------------------------------------------------------------%

strcpy(Buffer, SrcString) -->
    strncpy(Buffer ^ buffer, SrcString, Buffer ^ size).

% strncpy(DestBuffer, SrcString, Size)
:- pred strncpy(pointer, string, int, io, io).
:- mode strncpy(in, in, in, di, uo) is det.

:- pragma foreign_proc(c,
	strncpy(Buffer::in, Str::in, Size::in, IO0::di, IO::uo),
	[will_not_call_mercury, thread_safe, promise_pure], "
    strncpy(Buffer, Str, Size);
    IO = IO0;
").

%--------------------------------------------------------------------%

:- pragma foreign_proc(c,
	index(Index::in, Buffer::in, Char::out, IO0::di, IO::uo),
	[will_not_call_mercury, thread_safe, promise_pure], "
    Char = ((char *)Buffer)[Index];
    IO = IO0;
").

:- pragma foreign_proc(c,
	set(Index::in, Buffer::in, Char::in, IO0::di, IO::uo),
	[will_not_call_mercury, thread_safe, promise_pure], "
    ((char *)Buffer)[Index] = Char;
    IO = IO0;
").

:- pragma foreign_proc(c,
	offset(Index::in, Buffer::in, NewBuffer::out, IO0::di, IO::uo),
	[will_not_call_mercury, thread_safe, promise_pure], "
    NewBuffer = ((char *)Buffer) + Index;
    IO = IO0;
").

%--------------------------------------------------------------------%

:- end_module posix__buffer.

