%------------------------------------------------------------------------------%
% Copyright (C) 2001 The University of Melbourne.
% This file may only be copied under the terms of the GNU Library General
% Public License - see the file COPYING.LIB in the Mercury distribution.
%------------------------------------------------------------------------------%
%
% module: posix__environ.m
% main author: Michael Day <mikeday@bigpond.net.au>
%
%------------------------------------------------------------------------------%
:- module posix__environ.

:- interface.

:- import_module assoc_list.

:- pred environ(assoc_list(string), io, io).
:- mode environ(out, di, uo) is det.

%------------------------------------------------------------------------------%

:- implementation.

:- import_module list, std_util.

:- pragma c_header_code("
	#include <unistd.h>

	extern char **environ;
").

%------------------------------------------------------------------------------%

environ(Environ) -->
    get_environment(0, Environ).

:- pred get_environment(int, assoc_list(string), io, io).
:- mode get_environment(in, out, di, uo) is det.

get_environment(N, Environ) -->
    get_var(N, Var, Res),
    ( if { Res = 0 } then
	{ Environ = [] }
    else
	get_environment(N+1, Environ0),
	{ if sub_string_search(Var, "=", Index) then
	    Name = left(Var, Index),
	    Value = right(Var, length(Var) - Index - 1),
	    Environ = [Name-Value|Environ0]
	else
	    Environ = Environ0
	}
    ).

:- pred get_var(int, string, int, io, io).
:- mode get_var(in, out, out, di, uo) is det.

:- pragma c_code(get_var(N::in, Var::out, Res::out, IO0::di, IO::uo),
		[will_not_call_mercury], "
	Var = environ[N];
	Res = (Var != NULL);
	IO = IO0;
").

%------------------------------------------------------------------------------%

