:- module posix__hostent.		% Copyright (C) 2002 YesLogic Pty. Ltd.

:- interface.

:- type hostent
    --->    hostent(
		h_name :: string,
		h_aliases :: list(string),
		h_addr_list :: list(inet_addr)
	    ).

:- type hresult
    --->    ok(hostent)
    ;	    host_not_found
    ;	    no_address
    ;	    no_recovery
    ;	    try_again.

% gethostbyname(Name, Res)
:- pred gethostbyname(string, hresult, io, io).
:- mode gethostbyname(in, out, di, uo) is det.

%--------------------------------------------------------------------%

:- implementation.

:- pragma foreign_decl(c, "
#include <netdb.h>

extern int h_errno;
").

:- type hostent0.
:- pragma foreign_type(c, hostent0, "struct hostent *").

%--------------------------------------------------------------------%

:- pred h_name(hostent0, string, io, io).
:- mode h_name(in, out, di, uo) is det.

:- pragma foreign_proc(c, h_name(Hostent::in, Name::out, IO0::di, IO::uo),
	[will_not_call_mercury, promise_pure], % NOT THREAD SAFE
"
    Name = Hostent->h_name;
    IO = IO0;
").

:- pred h_aliases(hostent0, int, list(string), list(string), io, io).
:- mode h_aliases(in, in, in, out, di, uo) is det.

h_aliases(Hostent, I, Aliases0, Aliases) -->
    h_alias(Hostent, I, Alias, Res0),
    ( if { Res0 = 0 } then
	{ Aliases = Aliases0 }
    else
	h_aliases(Hostent, I+1, [Alias|Aliases0], Aliases)
    ).

:- pred h_alias(hostent0, int, string, int, io, io).
:- mode h_alias(in, in, out, out, di, uo) is det.

:- pragma foreign_proc(c,
	h_alias(Hostent::in, I::in, Alias::out, Res::out, IO0::di, IO::uo),
	[will_not_call_mercury, promise_pure], % NOT THREAD SAFE
"
    if (Hostent->h_aliases[I] != NULL) {
	MR_make_aligned_string_copy(Alias, Hostent->h_aliases[I]);
	Res = 1;
    } else {
	Alias = NULL;
	Res = 0;
    }
    IO = IO0;
").

:- pred h_addrs(hostent0, int, list(inet_addr), list(inet_addr), io, io).
:- mode h_addrs(in, in, in, out, di, uo) is det.

h_addrs(Hostent, I, Addrs0, Addrs) -->
    h_addr(Hostent, I, Addr, Res0),
    ( if { Res0 = 0 } then
	{ Addrs = Addrs0 }
    else
	h_addrs(Hostent, I+1, [Addr|Addrs0], Addrs)
    ).

:- pred h_addr(hostent0, int, inet_addr, int, io, io).
:- mode h_addr(in, in, out, out, di, uo) is det.

:- pragma foreign_proc(c,
	h_addr(Hostent::in, I::in, Addr::out, Res::out, IO0::di, IO::uo),
	[will_not_call_mercury, promise_pure], % NOT THREAD SAFE
"
    if (Hostent->h_addr_list[I] != NULL) {
	Addr = *((int *)Hostent->h_addr_list[I]);
	Res = 1;
    } else {
	Addr = 0;
	Res = 0;
    }
    IO = IO0;
").

%--------------------------------------------------------------------%

gethostbyname(Name, Res) -->
    gethostbyname0(Name, Hostent0, Res0),
    ( if { Res0 = 0 } then
	h_name(Hostent0, CanName),
	h_aliases(Hostent0, 0, [], Aliases),
	h_addrs(Hostent0, 0, [], Addrs),
	{ Res = ok(hostent(CanName, Aliases, Addrs)) }
    else if { Res0 = 1 } then { Res = host_not_found }
    else if { Res0 = 2 } then { Res = try_again }
    else if { Res0 = 3 } then { Res = no_recovery }
		         else { Res = no_address }
    ).

:- pred gethostbyname0(string, hostent0, int, io, io).
:- mode gethostbyname0(in, out, out, di, uo) is det.

:- pragma foreign_proc(c,
	gethostbyname0(Name::in, Hostent::out, Res::out, IO0::di, IO::uo),
	[will_not_call_mercury, promise_pure], % NOT THREAD SAFE
"
    Hostent = gethostbyname(Name);
    Res = h_errno;
    IO = IO0;
").

%--------------------------------------------------------------------%

:- end_module posix__hostent.

